@@ -151,12 +151,13 @@ class LabelStudioMLManager(object):
         project_model_dir = os.path.join(cls.model_dir, project or '')
         if not os.path.exists(project_model_dir):
             return
-        subdirs = list(filter(lambda d: d.isdigit(), os.listdir(project_model_dir)))
-        if subdirs:
-            last_version = str(list(sorted(map(int, subdirs)))[-1])
-            job_result_file = os.path.join(project_model_dir, last_version, 'job_result.json')
+
+        # sort directories by decreasing timestamps
+        for subdir in reversed(sorted(map(int, filter(lambda d: d.isdigit(), os.listdir(project_model_dir))))):
+            job_result_file = os.path.join(project_model_dir, str(subdir), 'job_result.json')
             if not os.path.exists(job_result_file):
-                return
+                logger.error('The latest job result file ' + job_result_file + ' doesn\'t exist')
+                continue
             with open(job_result_file) as f:
                 return json.load(f)
 
@@ -196,8 +197,10 @@ class LabelStudioMLManager(object):
     @classmethod
     def fetch(cls, project=None, label_config=None, force_reload=False, **kwargs):
         if cls.without_redis():
+            logger.debug('Fetch ' + project + ' from local directory')
             job_result = cls._get_latest_job_result_from_workdir(project) or {}
         else:
+            logger.debug('Fetch ' + project + ' from Redis')
             job_result = cls._get_latest_job_result_from_redis(project) or {}
         train_output = job_result.get('train_output')
         version = job_result.get('version')
@@ -269,11 +272,11 @@ class LabelStudioMLManager(object):
         data = list(data_iter)
         data_file = os.path.join(workdir, 'train_data.json')
         with io.open(data_file, mode='w') as fout:
-            json.dump(data, fout, ensure_ascii=False, indent=2)
+            json.dump(data, fout, ensure_ascii=False)
 
         info_file = os.path.join(workdir, 'train_data_info.json')
         with io.open(info_file, mode='w') as fout:
-            json.dump({'count': len(data)}, fout, indent=2)
+            json.dump({'count': len(data)}, fout)
 
     @classmethod
     def train_script_wrapper(
@@ -285,6 +288,11 @@ class LabelStudioMLManager(object):
             initialization_params = initialization_params or {}
             cls.initialize(**initialization_params)
 
+        # fetching the latest model version before we generate the next one
+        t = time.time()
+        m = cls.fetch(project, label_config)
+        m.is_training = True
+
         version = cls._generate_version()
 
         if cls.model_dir:
@@ -305,9 +313,6 @@ class LabelStudioMLManager(object):
             data_stream, snapshot = tee(data_stream)
             cls.create_data_snapshot(snapshot, workdir)
 
-        t = time.time()
-        m = cls.fetch(project, label_config)
-        m.is_training = True
         try:
             train_output = m.model.fit(data_stream, workdir, **train_kwargs)
             if cls.without_redis():
@@ -341,7 +346,7 @@ class LabelStudioMLManager(object):
             cls.train_script_wrapper,
             args=(project, label_config, train_kwargs, cls.get_initialization_params()),
             job_timeout='365d',
-            ttl=-1,
+            ttl=None,
             result_ttl=-1,
             failure_ttl=300,
             meta={'project': project},
