@@ -10,6 +10,7 @@ from operator import itemgetter
 from xml.etree import ElementTree
 from uuid import uuid4
 from copy import deepcopy
+from datetime import datetime
 
 from label_studio_converter import Converter
 
@@ -20,6 +21,7 @@ from label_studio.utils.models import ProjectObj, MLBackend
 from label_studio.utils.exceptions import ValidationError
 from label_studio.utils.io import find_file, delete_dir_content, json_load
 from label_studio.utils.validation import is_url
+from label_studio.utils.functions import get_full_hostname
 from label_studio.tasks import Tasks
 from label_studio.storage import create_storage, get_available_storage_names
 
@@ -405,7 +407,7 @@ class Project(object):
         sampling = self.config.get('sampling', 'sequential')
 
         # Tasks are ordered ascending by their "id" fields. This is default mode.
-        task_iter = filter(lambda i: i not in self.target_storage, self.source_storage.ids())
+        task_iter = filter(lambda i: i not in completed_tasks_ids, self.source_storage.ids())
         if sampling == 'sequential':
             task_id = next(task_iter, None)
             if task_id is not None:
@@ -460,10 +462,9 @@ class Project(object):
             num=len(completions), output_dir=self.config["output_dir"]))
         return sorted(completions)
 
-    def get_completed_at(self, task_ids):
-        """ Get completed time for list of task ids
+    def get_completed_at(self):
+        """ Get completed time for tasks
 
-        :param task_ids: list of task ids
         :return: list of string with formatted datetime
         """
         times = {}
@@ -477,6 +478,22 @@ class Project(object):
                 times[id] = timestamp_to_local_datetime(latest_time).strftime('%Y-%m-%d %H:%M:%S')
         return times
 
+    def get_skipped_status(self):
+        """ Get skipped status for tasks: returns skipped completion number for task
+
+        :return: list of int
+        """
+        items = {}
+        for _, data in self.target_storage.items():
+            id = data['id']
+            try:
+                flag = sum([completion.get('skipped', False) for completion in data['completions']])
+            except Exception as exc:
+                items[id] = -1
+            else:
+                items[id] = flag
+        return items
+
     def get_task_with_completions(self, task_id):
         """ Get task with completions
 
@@ -551,9 +568,10 @@ class Project(object):
                     continue
                 predictions = ml_backend.make_predictions(task, self)
                 predictions['created_by'] = ml_backend.model_name
+                predictions['created_date'] = datetime.now().isoformat()
                 task['predictions'].append(predictions)
         except Exception as exc:
-            logger.debug(exc)
+            logger.debug(exc, exc_info=True)
         return task
 
     def train(self):
@@ -734,6 +752,10 @@ class Project(object):
             config['host'] = args.host
         if args.allow_serving_local_files:
             config['allow_serving_local_files'] = True
+        if args.key_file and args.cert_file:
+            config['protocol'] = 'https://'
+            config['cert'] = args.cert_file
+            config['key'] = args.key_file
 
         # create config.json
         config_json = 'config.json'
@@ -873,6 +895,8 @@ class Project(object):
             'source_storage': {'readable_path': project.source_storage.readable_path},
             'available_storages': available_storages,
             'source_syncing': self.source_storage.is_syncing,
-            'target_syncing': self.target_storage.is_syncing
+            'target_syncing': self.target_storage.is_syncing,
+            'data_types': self.data_types,
+            'label_config_line': self.label_config_line
         }
         return output
