@@ -1,8 +1,8 @@
 import os
-import json
 import re
 import logging
 import threading
+import ujson as json
 
 from abc import ABC, abstractmethod
 from datetime import datetime, timedelta
@@ -245,7 +245,7 @@ class CloudStorage(BaseStorage):
     def _save_ids(self):
         if self._save_to_file_enabled:
             with open(self._ids_file, mode='w') as fout:
-                json.dump(self._ids_keys_map, fout, indent=2)
+                json.dump(self._ids_keys_map, fout)
 
     @abstractmethod
     def _get_value(self, key):
@@ -261,21 +261,33 @@ class CloudStorage(BaseStorage):
         else:
             return self._get_value(key)
 
-    def get(self, id):
+    def _get_key_by_id(self, id):
         item = self._ids_keys_map.get(id)
-        if item:
-            try:
-                key = item['key'].split(self.key_prefix, 1)[-1]
-                data = self.get_data(key)
-            except Exception as exc:
-                # return {'error': True, 'message': str(exc)}
-                logger.error(str(exc), exc_info=True)
-                return
-            if 'data' in data:
-                data['id'] = id
-                return data
-            else:
-                return {'data': data, 'id': id}
+        if not item:
+            # selected id not found in fetched keys
+            return
+        item_key = item['key']
+        if not item_key.startswith(self.key_prefix + self.prefix):
+            # found key not from current storage
+            return
+        return item_key
+
+    def get(self, id):
+        item_key = self._get_key_by_id(id)
+        if not item_key:
+            return
+        try:
+            key = item_key.split(self.key_prefix, 1)[-1]
+            data = self.get_data(key)
+        except Exception as exc:
+            # return {'error': True, 'message': str(exc)}
+            logger.error(str(exc), exc_info=True)
+            return
+        if 'data' in data:
+            data['id'] = id
+            return data
+        else:
+            return {'data': data, 'id': id}
 
     def _id_to_key(self, id):
         if not isinstance(id, str):
@@ -292,18 +304,23 @@ class CloudStorage(BaseStorage):
     def _set_value(self, key, value):
         pass
 
-    def set(self, id, value):
+    def _pre_set(self, id, value):
         if self.prefix:
             key = self.prefix + '/' + str(id)
         else:
             key = str(id)
         full_key = self.key_prefix + key
-        logger.debug('Create ' + full_key + ' in ' + self.readable_path)
         self._set_value(key, value)
         self._ids_keys_map[id] = {'key': full_key, 'exists': True}
         self._keys_ids_map[full_key] = id
         self._selected_ids.append(id)
+        return full_key
+
+    def set(self, id, value):
+        full_key = self._pre_set(id, value)
         self._save_ids()
+        logger.debug('Create ' + full_key + ' in ' + self.readable_path)
+
         if self.create_local_copy:
             self._create_local(id, value)
 
@@ -314,7 +331,7 @@ class CloudStorage(BaseStorage):
         local_file = os.path.join(self.objects_dir, str(id) + '.json')
         logger.debug('Creating local copy in file ' + local_file)
         with open(local_file, mode='w', encoding='utf8') as fout:
-            json.dump(value, fout, indent=2)
+            json.dump(value, fout)
 
     def max_id(self):
         return max(self._ids_keys_map.keys(), default=-1)
@@ -347,13 +364,16 @@ class CloudStorage(BaseStorage):
 
     def iter_full_keys(self):
         for key in self._get_objects():
+
+            if self.regex is not None and not self.regex.match(key):
+                logger.debug(key + ' is skipped by regex filter')
+                continue
+
             try:
                 self._validate_object(key)
             except Exception as exc:
                 continue
-            if not self.regex.match(key):
-                logger.debug(key + ' is skipped by regex filter')
-                continue
+
             yield self.key_prefix + key
 
     def _sync(self):
@@ -365,12 +385,18 @@ class CloudStorage(BaseStorage):
         new_ids_keys_map = {}
         new_keys_ids_map = {}
 
-        for key in self.iter_full_keys():
-            if key not in self._keys_ids_map:
-                id = new_id
-                new_id += 1
-            else:
-                id = self._keys_ids_map[key]
+        full = set(self.iter_full_keys())
+        intersect = full & set(self._keys_ids_map)
+        exclusion = full - intersect
+
+        for key in exclusion:
+            id = new_id
+            new_id += 1
+            new_ids_keys_map[id] = {'key': key, 'exists': True}
+            new_keys_ids_map[key] = id
+
+        for key in intersect:
+            id = self._keys_ids_map[key]
             new_ids_keys_map[id] = {'key': key, 'exists': True}
             new_keys_ids_map[key] = id
 
@@ -396,7 +422,8 @@ class CloudStorage(BaseStorage):
         return len(self._ids_keys_map) == 0
 
     def __contains__(self, id):
-        return id in self._ids_keys_map
+        item_key = self._get_key_by_id(id)
+        return item_key is not None
 
     def remove(self, key):
         raise NotImplementedError
