@@ -3,7 +3,7 @@ import logging
 
 from google.cloud import storage
 
-from .base import CloudStorage, BaseForm, StringField, BooleanField, Optional
+from .base import CloudStorage, BaseStorageForm, StringField, BooleanField, Optional
 
 
 logger = logging.getLogger(__name__)
@@ -13,9 +13,13 @@ class GCSStorage(CloudStorage):
 
     description = 'Google Cloud Storage'
 
+    @property
+    def url_prefix(self):
+        return 'gs://'
+
     @property
     def readable_path(self):
-        return 'gs://' + self.path + '/' + self.prefix
+        return self.url_prefix + self.path + '/' + self.prefix
 
     def _get_client(self):
         client = storage.Client()
@@ -39,10 +43,6 @@ class GCSStorage(CloudStorage):
         blob_str = blob.download_as_string()
         return json.loads(blob_str)
 
-    def _get_value_url(self, key):
-        data_key = self.data_key if self.data_key else self.default_data_key
-        return {data_key: 'gs://' + self.path + '/' + key}
-
     def _set_value(self, key, value):
         if not isinstance(value, str):
             value = json.dumps(value)
@@ -51,13 +51,14 @@ class GCSStorage(CloudStorage):
         blob.upload_from_string(value)
 
 
-class GCSCompletionsStorageForm(BaseForm):
+class GCSCompletionsStorageForm(BaseStorageForm):
     prefix = StringField('Prefix', [Optional()], description='GCS Bucket prefix')
     create_local_copy = BooleanField('Create local copy', description='Create a local copy on your disk', default=True)
 
     bound_params = dict(
         prefix='prefix',
-        create_local_copy='create_local_copy'
+        create_local_copy='create_local_copy',
+        **BaseStorageForm.bound_params
     )
 
 
