@@ -1,6 +1,7 @@
 # Import tasks from json, csv, zip, txt and more
 
 import os
+import io
 import csv
 import hashlib
 import shutil
@@ -9,6 +10,7 @@ import rarfile
 import logging
 import tempfile
 import pandas as pd
+import htmlmin
 try:
     import ujson as json
 except:
@@ -19,7 +21,7 @@ from urllib.request import urlopen
 
 from .exceptions import ValidationError
 from .misc import Settings
-from label_studio.utils.functions import HOSTNAME
+from label_studio.utils.functions import get_full_hostname, get_web_protocol
 
 
 settings = Settings
@@ -45,16 +47,30 @@ def tasks_from_file(filename, file, project):
                 tasks = json.loads(raw_data)
             except TypeError:
                 tasks = json.loads(raw_data.decode('utf8'))
-        else:
-            # save file to disk
+
+        # upload file via drag & drop
+        elif len(project.data_types) > 1:
+            raise ValidationError('Your label config has more than one data keys, direct file upload supports only'
+                                  ' one data key. To import data with multiple data keys use JSON or CSV')
+        # convert html file to json task
+        elif filename.endswith('.html') or filename.endswith('.htm') or filename.endswith('.xml'):
             data = file.read()
-            upload_dir = os.path.join(project.name, 'upload')
+            body = htmlmin.minify(data.decode('utf8'), remove_all_empty_space=True)
+            tasks = [{'data': {settings.UPLOAD_DATA_UNDEFINED_NAME: body}}]
+        # hosting for file
+        else:
+            # read as text or binary file
+            data = open(filename, 'rb').read() if isinstance(file, io.TextIOWrapper) else file.read()
+
+            upload_dir = os.path.join(project.path, 'upload')
             os.makedirs(upload_dir, exist_ok=True)
-            filename = hashlib.md5(data).hexdigest() + '-' + filename
+            filename = hashlib.md5(data).hexdigest() + '-' + os.path.basename(filename)
             path = os.path.join(upload_dir, filename)
             open(path, 'wb').write(data)
             # prepare task
-            tasks = [{'data': {settings.UPLOAD_DATA_UNDEFINED_NAME: HOSTNAME + '/data/upload/' + filename}}]
+
+            path = get_full_hostname() + '/data/upload/' + filename
+            tasks = [{'data': {settings.UPLOAD_DATA_UNDEFINED_NAME: path}}]
 
     except Exception as exc:
         raise ValidationError('Failed to parse input file ' + filename + ': ' + str(exc))
